// ALista.cs - Apstraktna klasa listi.

using System.Text;

namespace Zbirke {
  public abstract class ALista: IZbirka {
    protected Elem prvi, posl;              // Poetak i kraj liste.
    protected int n;                        // Duina liste.
  
    protected class Elem {                  // Element liste:
      public int broj;                      // - sadrani broj,
      public Elem sled;                     // - sledei element,
      public Elem(int b) { broj = b; }      // - inicijalizacija.
    }
  
    protected Elem Nadji(int i) {           // Nalaenje elementa.
      if (i<0 || i>=n) throw new GreIndeks(i);
      Elem tek = prvi; while (i-- > 0) tek = tek.sled;
      return tek;
    }

    public int Vel { get{ return n; } }     // Veliina zbirke.

    public abstract IZbirka Dodaj(int b);   // Dodavanje elementa.

    public abstract int this[int i] { get; set; } // Pristupanje elementu.

    public IZbirka Brisi(int i) {           // Izbacivanje elementa.
      Elem pret = Nadji(i-1);
      if (pret.sled == null) throw new GreIndeks(i);
      pret.sled = pret.sled.sled; n--;
      return this;
    }
  
    public IZbirka Isprazni()               // Pranjenje zbirke.
      { prvi = posl = null; n = 0; return this; }

    public abstract IIterator Iterator();   // Stvaranje iteratora.

    public override string ToString() {     // Tekstualni opis zbirke.
      StringBuilder sb = new StringBuilder("[");
      for (Elem tek=prvi; tek!=null; tek=tek.sled) {
        if (tek != prvi) sb.Append(',');
        sb.Append(tek.broj);
      }
      return sb.Append(']').ToString();
    }
  
    // UGNEENA KLASA ITERATORA ZA LISTE.
    protected abstract class AIterAL: IIterator {
      protected ALista aLista;
      protected Elem tek, pret=null;        // Tekui i prethodni element.

      public AIterAL(ALista al)             // Inicijalizacija.
        { aLista = al; tek = aLista.prvi; }

      public IIterator NaPocetak()          // Pomeranje na poetak zbirke.
        { tek = aLista.prvi; pret = null; return this; }
  
      public IIterator NaSledeci() {        // Pomeranje na sledei element.
        if (tek != null) {pret = tek; tek = tek.sled;}
        return this;
      }
  
      public bool ImaTek                    // Ima li tekueg elementa.
        { get { return tek != null; } }

      public abstract int Tek { get; set; } // Pristupanje tekuem elementu.
        
      public IIterator BrisiTek() {         // Brisanje tekueg elementa.
        if (!ImaTek) throw new GreNemaTek();
        pret.sled = tek = tek.sled; aLista.n--;
        return this;
      }
    }
  }
}
