// ANiz.cs - Apstraktna klasa nizova.

namespace Zbirke {
  using System.Text;

  public abstract class ANiz: IZbirka {
    protected int[] niz;                          // Elementi niza.
    protected int n;                              // Broj elemenata niza.
  
    public ANiz() { niz = new int [5]; }          // Inicijalizacija.
  
    public int Vel { get { return n; } }          // Veliina zbirke.

    public abstract IZbirka Dodaj(int b);         // Dodavanje elementa.
  
    protected void povecaj() {                    // Poveavanje kapaciteta
      int k = (n>50 ? n/10 : 5);                  //   (10%, ali bar za 5).
      int[] pom = new int[n+k];
      for (int i=0; i<n; pom[i]=niz[i++]);
      niz = pom;
    }

    public abstract int this[int i] { get; set; } // Pristupanje elementu.

    public IZbirka Brisi(int i) {                 // Izbacivanje elementa.
      if (i<0 || i>=n) throw new GreIndeks(i);
      int m = niz.Length - n;
      if (m > niz.Length/5 && m > 5) {         // - ako je bar 20% slobodno,
        m = (niz.Length>10 ? niz.Length/10 : 5);  //   skrati niz za 10%,
        int[] pom = new int [niz.Length - m];     //   ali bar za 5.
        for (int j=0, k=0; j<n; j++)
          if (j != i) pom[k++] = niz[j];
        niz = pom;
      } else
        while (i < n-1) niz[i] = niz[++i];
      n--;
      return this;
    }
  
    public IZbirka Isprazni()                     // Pranjenje zbirke.
      { niz = new int [5]; n = 0; return this; }
  
    public abstract IIterator Iterator();         // Stvaranje iteratora.

    public override string ToString() {           // Tekstualni opis zbirke.
      StringBuilder sb = new StringBuilder("[");
      for (int i=0; i<n; i++) {
        if (i > 0) sb.Append(',');
        sb.Append(niz[i]);
      }
      return sb.Append(']').ToString();
    }
  
    // UGNEENA KLASA ITERATORA ZA NIZOVE.
    protected abstract class AIterAN: IIterator {
      protected ANiz aNiz;                  // Vlasnika zbirka.
      protected int tek = 0;                // Indeks tekueg elementa.

      protected AIterAN(ANiz an) { aNiz = an; } // Inicijalizacija.

      public IIterator NaPocetak()          // Pomeranje na poetak zbirke.
        { tek = 0; return this; }
  
      public IIterator NaSledeci()          // Pomeranje na sledei element.
        { tek++; return this; }
  
      public bool ImaTek                    // Ima li tekueg elementa?
        { get { return tek < aNiz.n; } }
  
      public abstract int Tek { get; set; } // Pristupanje tekuem elementu.

      public IIterator BrisiTek() { // Brisanje tekueg elementa.
        if (!ImaTek) throw new GreNemaTek();
        aNiz.Brisi(tek);
        return this;
      }
    }
  }
}
