// Citaj.cs itanje podataka standardnih tipova iz ulaznog toka,
//          iz datoteke i sa standardog ulaza.

using System;
using System.IO;
using System.Text;

public class Citaj {
  private TextReader ut;              // Ulazni tok iz kojeg se ita.

  public Citaj(TextReader uut) { ut = uut; } // Inicijalizacija
                                             //   ulaznim tokom.
  public Citaj(string ime)           // Otvaranje datoteke.
    { ut = new StreamReader(ime); }

  public bool EndS()                 // Da li je kraj toka?
    { return ut.Peek() == -1; }

  public char GetChS()               // Dohvatanje sledeeg znaka.
    { return (char)ut.Read(); }

  public char CharS() {              // itanje jednog (nebelog) znaka.
    char zn = '\0';
    while (char.IsWhiteSpace(zn = GetChS()));
    return zn;
  }

  public string StringS() {          // itanje jedne rei.
    StringBuilder sb = new StringBuilder(CharS().ToString());
    int zn = '\0';
    while ((zn=ut.Peek())!=-1 && !char.IsWhiteSpace((char)zn))
      sb.Append((char)ut.Read());
    return sb.ToString();
  }

  public string LineS()              // itanje jednog reda teksta.
    { return ut.ReadLine(); }

  public byte ByteS()                // itanje podatka tipa byte.
    { return byte.Parse(StringS()); }

  public sbyte SByteS()              // itanje podatka tipa sbyte.
    { return sbyte.Parse(StringS()); }

  public short ShortS()              // itanje podatka tipa short.
    { return short.Parse(StringS()); }
 
  public ushort UShortS()            // itanje podatka tipa ushort.
    { return ushort.Parse(StringS()); }

  public int IntS()                  // itanje podatka tipa int.
    { return int.Parse(StringS()); }

  public uint UIntS()                // itanje podatka tipa uint.
    { return uint.Parse(StringS()); }

  public long LongS()                // itanje podatka tipa long.
    { return long.Parse(StringS()); }

  public ulong ULongS()              // itanje podatka tipa ulong.
    { return ulong.Parse(StringS()); }

  public float FloatS()              // itanje podatka tipa float.
    { return float.Parse(StringS()); }

  public double DoubleS()            // itanje podatka tipa double.
    { return double.Parse(StringS()); }

  public decimal DecimalS()          // itanje podatka tipa decimal.
    { return decimal.Parse(StringS()); }

  // PODRKA ZA ITANJE SA STANDARDOG ULAZA.

  private static Citaj gl = new Citaj(Console.In); // Predstavnik standardog
                                                   //   ulaza.
  public static bool   End   () { return gl.EndS   (); } // Varijante metoda
  public static char   GetCh () { return gl.GetChS (); } //   koje itaju s
  public static char   Char  () { return gl.CharS  (); } //   standardog ulaza,
  public static string String() { return gl.StringS(); }
  public static string Line  () { return gl.LineS  (); }
  public static byte   Byte  () { return gl.ByteS  (); }
  public static sbyte  SByte () { return gl.SByteS (); }
  public static short  Short () { return gl.ShortS (); }
  public static ushort UShort() { return gl.UShortS(); }
  public static int    Int   () { return gl.IntS   (); }
  public static uint   UInt  () { return gl.UIntS  (); }
  public static long   Long  () { return gl.LongS  (); }
  public static ulong  ULong () { return gl.ULongS (); }
  public static float  Float () { return gl.FloatS (); }
  public static double Double() { return gl.DoubleS(); }
  public static decimal Decimal() { return gl.DecimalS(); }

}  // class Citaj
	