// Kaledar.cs - Klasa kalendara za jednu godinu.

using System;
using System.Text;

namespace Kalendari {
  public class Kalendar {
    private static readonly int[,] brDana = { // Broj dana po mesecima.
      {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
    };
    private int god;                       // Predstavljena godina.
    private Dan[,] dani = new Dan[12,31];  // Dani u nedelji za celu godinu.

    public Kalendar(int g) { PostaviGod(g); } // Inicijalizacija.

    public static bool Prestupna(int g)       // Da li je prestupna godina?
      { return g%4==0 && g%100!=0 || g%400==0; }
 
    public void PostaviGod(int g) {                     // Menjanje godine:
      god = g;
      int pr = Prestupna(god) ? 1 : 0;
      Dan dan = default(Dan);
      switch (new DateTime(g,1,1).DayOfWeek) {          // - dan u nedelji
        case DayOfWeek.Monday:    dan = Dan.Pon; break; //   za prvi jan.,
        case DayOfWeek.Tuesday:   dan = Dan.Uto; break;
        case DayOfWeek.Wednesday: dan = Dan.Sre; break;
        case DayOfWeek.Thursday:  dan = Dan.Cet; break;
        case DayOfWeek.Friday:    dan = Dan.Pet; break;
        case DayOfWeek.Saturday:  dan = Dan.Sub; break;
        case DayOfWeek.Sunday:    dan = Dan.NED; break;
      }
      for (Mesec m=Mesec.Jan; m<=Mesec.Dec; m++) {      // - sastavljanje
        for (int d=0; d<brDana[pr,(int)m-1];            //    kalendara.
             d++, dan=dan<Dan.NED ? dan+1 : Dan.Pon)
          dani[(int)m-1,d] = dan;
      }
    }

    public Dan DanUNedelji(int d, int m)    // Dan u nedelji datog datuma.
      { return dani[m-1,d-1]; }
	
    public override string ToString() {     // Tekstualni prikaz kalendara.
      StringBuilder sb = new StringBuilder();
      sb.Append(string.Format("Kalendar za {0:d}. godinu.\n\n   |", god));
      for (Mesec m=Mesec.Jan; m<=Mesec.Dec; m++)
        sb.Append(string.Format("{0,4}", m));
      sb.Append("\n---+");
      for (int i=0; i<12; i++) sb.Append("----");
      sb.Append("\n");
      for (int d=0; d<31; d++) {
        sb.Append(string.Format("{0,2:d} |", d+1));
        for (int m=0; m<12; m++)
          sb.Append(string.Format("{0,4}",
             dani[m,d]!=default(Dan) ? dani[m,d].ToString() : ""
          ));
        sb.Append("\n");
      }
      return sb.ToString();
    }
  }
}
