// Kamion.cs - Klasa kamiona.

namespace Prevoz {
  public class Kamion: IMerljiv {
    private double sopTezina, nosivost; // Sopstvena teina i nosivost.
    private double teret = 0;           // Teina tereta.
    private Elem prvi, posl;            // Prvi i poslednji element liste.
  
    private class Elem {                // Element liste paketa:
      public Paket pak;                 // - sadrani paket,
      public Elem sled = null;          // - sledei element liste, 
      public Elem(Paket p) { pak = p; } // - inicijalizacija.
    }

    public Kamion(double sT, double n)  // Inicijalizacija.
      { sopTezina = sT; nosivost = n; }
  
    public Kamion Dodaj(Paket p) {      // Dodavanje paketa.
      if (teret+p.Tezina > nosivost) throw new GreTeret();
      Elem novi = new Elem(p);
      if (prvi == null) prvi = novi; else posl.sled = novi;
      posl = novi;
      teret += p.Tezina;
      return this;
    }

    public double Tezina                // Teina kamiona.
      { get { return sopTezina + teret; } }

    public double Vrednost { get {      // Vrednost tereta.
      double v = 0;
      for (Elem tek=prvi; tek!=null; tek=tek.sled)
        v += tek.pak.Vrednost;
      return v;
    }}

    public override string ToString() { // Tekstualni opis.
      string s = "Kamion(" + nosivost + "," + sopTezina + ","
                           + Tezina + "," + Vrednost + ")";
      for (Elem tek=prvi; tek!=null; tek=tek.sled)
        s += "\n" + tek.pak;
      return s;
    }
  }
}
