// Matr.cs - Klasa matrica realnih brojeva.

using System;
using System.Text;

namespace Matrica {
  public class Matr: ICloneable {
    double[,] mat;                                      // Elementi matrice.

    public Matr(int m, int n) { mat = new double[m,n]; }// Inicijalizacija.

    public Matr(): this(10,10) {}

    public int Vrs { get { return mat.GetLength(0); } } // Broj vrsta.

    public int Kol { get { return mat.GetLength(1); } } // Broj kolona.

    private void proveriInd(int i, int j) {             // Provera indeksa.
      if (i<0 || i>=Vrs || j<0 || j>=Kol)
        throw new GreMatInd(i, j);
    }

    public double this[int i, int j] {            // Pristupanje elementu:
      get { proveriInd(i, j); return mat[i,j]; }  // - dohvatanje,
      set { proveriInd(i, j); mat[i,j] = value; } // - postavljanje.
    }

    public object Clone() {                       // Stvaranje kopije.
      Matr m = (Matr)MemberwiseClone();
      m.mat = (double[,])mat.Clone();
      return m;
    }

    private static void proveriDim(Matr m1, Matr m2) { // Provera dimenzija.
      if (m1.Vrs != m2.Vrs || m1.Kol != m2.Kol ) 
        throw new GreMatDim(m1, m2);
    }
 
    public static Matr operator+(Matr m1, Matr m2) {   // Zbir matrica.
      proveriDim(m1, m2);
      Matr m = new Matr(m1.Vrs, m1.Kol);
      for (int i=0; i<m1.Vrs; i++)
        for (int j=0; j<m1.Kol; j++)
          m.mat[i,j] = m1.mat[i,j] + m2.mat[i,j];
      return m;
    }

    public static Matr operator-(Matr m1, Matr m2) {   // Razlika matrica.
      proveriDim(m1, m2);
      Matr m = new Matr(m1.Vrs, m1.Kol);
      for (int i=0; i<m1.Vrs; i++)
        for (int j=0; j<m1.Kol; j++)
          m.mat[i,j] = m1.mat[i,j] - m2.mat[i,j];
      return m;
    }

    public static Matr operator*(Matr m1, Matr m2) {   // Proizvod matrica.
      if (m1.Kol != m2.Vrs) throw new GreMatDim(m1, m2);
      Matr m = new Matr(m1.Vrs, m2.Kol);
      for (int i=0; i<m1.Vrs; i++)
        for (int k=0; k<m2.Kol; k++)
          for (int j=0; j<m2.Vrs; j++)
            m.mat[i,k] += m1.mat[i,j] * m2.mat[j,k];
      return m;
    }

    public Matr Citaj() {                              // itanje matrice.
      Console.Write("Vrs, kol? ");
      int vrs = global::Citaj.Int(), kol = global::Citaj.Int();
      mat = new double [vrs,kol];
      for (int i=0; i<vrs; i++) {
        Console.Write("Vrsta " + i + "? ");
        for (int j=0; j<kol; mat[i,j++]=global::Citaj.Double());
      }
      return this;
    }

    public override string ToString() { // Sastavljanje tekstualnog opisa.
      StringBuilder sb = new StringBuilder();
      for (int i=0; i<Vrs; i++){
        for (int j=0; j<Kol; sb.Append(mat[i,j++]).Append(' '));
        sb.Append('\n');
      }
      return sb.ToString();
    }
  }
}
