// Matrica2.cs - Klasa matrica numerikih podataka.

using System.Threading;
using System.Text;

namespace Aritmetika {
  public class Matrica: INumericko {
    private INumericko[,] matr;                 // Elementi matrice.
    private int m, n;                           // Dimenzije matrice.
 
    public Matrica(int mm, int nn)              // Inicijalizacija.
      { matr = new INumericko [m=mm,n=nn]; }

    public INumericko this[int i, int j] { set {  // Postavljanje elementa.
      if (i<0 || i>=m || j<0 || j>=n) throw new GreNumInd();
      matr[i,j] = value; 
    }}

    public object Clone() {                     // Kopija matrice.
      Matrica rez = (Matrica)MemberwiseClone();
      rez.matr = (INumericko[,])matr.Clone();
      for (int i=0; i<m; i++) {
        for (int j=0; j<n; j++)
          rez.matr[i,j] = (INumericko)matr[i,j].Clone();
      }
      return rez;
    }

    public INumericko Dodaj(INumericko bb) {    // Dodavanje matrice.
      if (!(bb is Matrica)) throw new GreNumTip();
      Matrica b = (Matrica)bb;
      if (m!=b.m || n!=b.n) throw new GreNumDim();
      for (int i=0; i<m; i++)
        for (int j=0; j<n; j++)
          matr[i,j].Dodaj(b.matr[i,j]);
      return this;
    }

    // Aktivna klasa za izraunavanje jednog elementa proizvoda matrica:
    private class Mnozac {
      private Thread nit;            // - nit u klasi,
      private Matrica a, b;          // - matrice koje se mnoe,
      private int i, k;              // - vrsta prve i kolona druge matrice,
      private INumericko cik;        // - vrednost elementa rezultata,

      public Mnozac(Matrica aa, Matrica bb, int ii, int kk) { // - inicija-
        a = aa; b = bb; i = ii; k = kk;                       //   lizacija.
        (nit = new Thread(radi)).Start();
      }

      private void radi() {                     // - raunanje,
        try {
          cik = a.matr[i,0].Proizvod(b.matr[0,k]);
          for (int j=1; j<a.n; j++)
            cik.Dodaj(a.matr[i,j].Proizvod(b.matr[j,k]));
        } catch (GreNumericka) { cik = null; }
      }

      public INumericko Cik()                   // - dohvatanje rezultata.
        { nit.Join(); return cik; }
    } // class Mnozac

    public INumericko Proizvod(INumericko bb) { // Proizvod dve matrice:
      if (!(bb is Matrica)) throw new GreNumTip();
      Matrica b = (Matrica)bb;
      if (n != b.m) throw new GreNumDim();
      Matrica c = new Matrica(m, b.n);

      Mnozac[,] mno = new Mnozac[c.m,c.n];      // - stvaranje niti,
      for (int i=0; i<c.m; i++)
        for (int k=0; k<c.n; k++)
          mno[i,k] = new Mnozac(this, b, i, k);
      for (int i=0; i<c.m; i++)                 // - prikupljanje rezultata.
        for (int k=0; k<c.n; k++)
          c.matr[i,k] = mno[i,k].Cik();
      return c;
    }

    public override string ToString() {         // Tekstualni opis.
      StringBuilder sb = new StringBuilder();
      for (int i=0; i<m; i++)
        for (int j=0; j<n; j++) {
          sb.Append(matr[i,j])
            .Append(j<n-1 ? ' ' : '\n');
        }
      return sb.ToString();
    }
  }
}
