// Merac.cs - Klasa meraa fizikih veliina.

namespace Merenje {
  using System.Threading;

  public class Merac {
    private IVel vel;                      // Merena veliina.
    private int dt;                        // Vreme izmeu dva merenja.
    private double s = 0;                  // Zbir svih uzoraka.
    private int n = 0;                     // Broj uzoraka.
    private volatile bool radi = false;    // Da li je aktivan.
    private Thread nit;                    // Nit koja radi.
    private bool kraj = false;             // Da li treba da se zavri?

    public Merac(IVel mv, int d) {         // Inicijalizacija.
      vel = mv; dt = d;
      (nit = new Thread(akcija)).Start();
    }

    private void akcija() {                // Telo niti.
      try {
        while (!kraj) {
          lock (this) { while (!radi) Monitor.Wait(this); }
          Thread.Sleep(dt);
          lock (this) { s += vel.Vrednost; n++; }
        }
      } catch (ThreadInterruptedException) {}
    }

    public double Vr {get{ return vel.Vrednost; }} // Trenutna vrednost.

    public double SrVr { get { lock (this) {       // Srednja vrednost.
      if (n == 0) throw new GreNemaSrVr();
      return s / n;
    }}}

    public void Stani()                    // Privremeno zaustavljanje.
      { radi = false; }
 
    public void Kreni()                                // Nastavak rada.
      { lock (this) { s = n = 0; radi = true; Monitor.Pulse(this); } }

    public void Zavrsi() {                             // Zavretak niti.
      if (vel is PromVel) ((PromVel)vel).Prekini();
      kraj = true; nit.Interrupt();
    }

    public override string ToString () { lock (this) { // Tekstualni opis.
      string s = string.Format("{0:f1} {1}", vel.Vrednost, vel.Jedinica);
      try {
        s += string.Format(" ({0:f1} {1})", SrVr, vel.Jedinica);
      } catch (GreNemaSrVr ) {}
      return s;
    }}
  }
}
