// Polica.cs - Klasa polica za predmete.

namespace Police {
  using Predmeti1;
  using System.Text;

  public class Polica {
    private Predmet[] niz;                // Niz za smestanje predmeta.
    private double maxQ;                  // Nosivost police.
    private double q = 0;                 // Trenutna opterecenost police.

    public Polica(int kap, double Qmax) { // Inicijalizacija.
      niz = new Predmet [kap];
      maxQ = Qmax;
    }
 
    public Polica Stavi(Predmet p) {  // Stavljanje na prvo slobodno mesto.
      if (q + p.Q > maxQ)  throw new GrePolTezina(p);
      int i = 0; while (i<niz.Length && niz[i]!=null) i++;
      if (i == niz.Length) throw new GrePolPuna();
      niz[i] = p; q += p.Q;
      return this;
    }

    public Predmet Uzmi(int i) {                // Uzimanje sa datog mesta.
      if (i<0 || i>=niz.Length) throw new GrePolIndeks(i);
      if (niz[i] == null)       throw new GrePolPrazno(i);
      Predmet p = niz[i]; niz[i] = null; q -= p.Q; return p;
    }

    public Predmet this[int i] {                // Pristup datom mestu:
      set {                                     // - stavljanje predmeta,
        if (i<0 || i>=niz.Length) throw new GrePolIndeks(i);
        if (niz[i] != null)       throw new GrePolZauzeto(i);
        if (q + value.Q > maxQ)   throw new GrePolTezina(value);
        niz[i] = value; q += value.Q;
      }
      get {                                     // - dohvatanje predmeta.
        if (i<0 || i>=niz.Length) throw new GrePolIndeks(i);
        if (niz[i] == null)       throw new GrePolPrazno(i);
        return niz[i];
      }
    }

    public bool Prazno(int i)  {                // Da li je mesto prazno?
      if (i<0 || i>=niz.Length) throw new GrePolIndeks (i);
      return niz[i] == null;
    }

    public int Kapacitet { get { return niz.Length; } } // Kapacitet police.

    public int Slobodno {                       // Broj slobodnih mesta.
      get {
        int k = 0; foreach ( Predmet p in niz) if (p == null) k++;
        return k;
      }
    }

    public int Zauzeto                          // Broj zauzetih mesta.
      { get { return Kapacitet  - Slobodno ; } }

    public double Nosivost{get{ return maxQ; }} // Nosivost police.

    public double Teret { get { return q; } }   // Ukupan teret na polici.

    public double Jos {get{ return maxQ - q; }} // Dozvoljeni dodatni teret.

    public override string ToString() {         // Tekstualni opis.
      StringBuilder s = new StringBuilder();
      for (int i=0; i<niz.Length; i++)
        if (niz[i] != null)
          s.Append (i).Append (". ").Append(niz[i]).Append('\n');
      return s.ToString();
    }
 
    public Polica Prazni() {                    // Pranjenje police.
      for (int i=0; i<niz.Length; niz[i++]=null);
      q = 0; return this;
    }
  }
}
