// RacBroj.cs - Klasa racionalnih brojeva.

class RacBroj {

  // Statiki niz prvih nekoliko prostih brojeva.
  private const int maxProst = 10000;
  private static readonly long[]
    prosti =  new long [maxProst];

  static RacBroj() {                   // Generisanje niza prostih brojeva.
    int n = 1;                         //   (statiki konstruktor).
    prosti[0] = 2;
    for (int p=3; n<maxProst; p+=2) {
      bool prost = true;
      for (int j=0; prosti[j]*prosti[j]<=p; j++)
        if (p % prosti[j] == 0) { prost = false; break; }
      if (prost) prosti[n++] = p;
    }
  }

  private long a, b;                   // Deljenik i delitelj.

  private void sredi() {               // Sreivanje broja.
    if (a == 0) b = 1;
    else if (b != 0) {
      if (b < 0) { a = -a; b = -b; }
      long absA = a<0 ? -a : a;
      for (int i=0; i < maxProst &&
                    prosti[i]*prosti[i]<=absA && 
                    prosti[i]*prosti[i]<=b;      i++) {
        while (a % prosti[i] == 0 && b % prosti[i] == 0) { 
          a /= prosti[i]; b /= prosti[i]; 
        }
      }
    }
  }

  public RacBroj(long a=0, long b=1) { // Inicijalizacija.
    this.a = a; this.b = b;
    sredi();
  }
 
  public RacBroj Dodaj(RacBroj x) {    // Dodavanje. 
    a = a * x.b + x.a * b; b *= x.b; 
    sredi(); return this;
  }

  public RacBroj Oduzmi(RacBroj x) {   // Oduzimanje.
    a = a * x.b - x.a * b; b *= x.b; 
    sredi(); return this;
  }

  public RacBroj Pomnozi(RacBroj x) {  // Mnoenje.
    a *= x.a; b *= x.b; 
    sredi (); return this;
  }

  public RacBroj Podeli(RacBroj x) {   // Deljenje.
    a *= x.b; b *= x.a; 
    sredi(); return this;
  }

  public override string ToString() {  // Tekstualni opis.
    if (b == 0) return "GRESKA";
    if (a == 0) return "0";
    string rez = "";
    if (a / b != 0) rez += a / b;
    if (b != 1) {
      if (rez!="" && a>0) rez += "+";
      rez += a%b + "/" + b;
    }
    return rez;
  }
}
