// Radnja.cs - Klasa radnji.

namespace Trgovina2 {
  using Police;
  using Pokretni;
  using Radionica;
  using Predmeti1;
  using Liste;
  using System;

  public class Radnja {
    private Masina[] masine = new Masina [10];  // Maine za proizvodnju.
    private int brMas;                          // Broj maina.
    private Polica polica;                      // Polica s artiklima.
    private Casovnik casovnik = new Casovnik(); // Generator takta.
    private Lista kupci = new Lista();          // Kupci u radnji.
    private static Random sluc=new Random(); // Generator sluajnih brojeva.
 
    public Radnja(                       // Inicijalizacija:
                   double maxQ,          // - nosivost police,
                   int kap,              // - broj mesta na polici,
                   double maxUlaz,       // - najdue vreme do ulaska kupca,
                   double maxKup         // - najdue vreme kupovine.
                  ) {
      polica = new Polica(kap, maxQ);
      casovnik.Vreme += new Ulaz(this, maxUlaz).Proteklo;
      casovnik.Vreme += new Prodavac(this, maxKup).Proteklo;
    }

    public Radnja DodajMas(Masina mas)         // Nabavka maine.
      { masine[brMas++] = mas; return this; }
    
    public Radnja Zaposli(PokRadnik radnik) {  // Zapoljavanje radnika.
      casovnik.Vreme += radnik.Proteklo;
      radnik.Polica(polica);
      radnik.Rasporedi(masine[(int)(sluc.NextDouble() * brMas)]);
      Console.WriteLine("Zaposlen je radnik " + radnik);
      return this;
    }

    public Radnja otpusti(PokRadnik radnik)    // Otputanje radnika.
      { casovnik.Vreme -= radnik.Proteklo; return this; }

    public void Radi(double dt, double maxT)   // Radno vreme radnje.
      { casovnik.Radi(dt, maxT); }

    private class Ulaz: IPokretan { // KLASA ULAZA U RADNJU.
      private Radnja radnja;        // Vlasnika radnja.
      private double maxT;          // Najdue vreme do ulaska kupca.
      private double t;             // Preostalo vreme do narednog dogaaja.
      private Kupac  kup;           // Sledei kupac koji ulazi.
  
      public Ulaz(Radnja r, double tMax)       // Inicijalizacija.
        { radnja = r; maxT = tMax; }
 
      public IPokretan Proteklo(double dt) {   // Sledei korak rada.
        if ((t -= dt) <= 0) {
          if (kup != null) {
            radnja.kupci.Dodaj(kup);
            Console.WriteLine("Kupac  " + kup + " je usao");
          }
          int i = (int)(sluc.NextDouble() * radnja.brMas);
          kup = new Kupac(radnja.masine[i].Vrsta);
          t = sluc.NextDouble() * maxT;
        }
        return this;
      }
    } // class Ulaz
  
    private class Prodavac: IPokretan { // KLASA PRODAVCA.
      private Radnja radnja;        // Vlasnika radnja.
      private double maxT;          // Najdue vreme kupovine.
      private double t;             // Preostalo vreme do narednog dogaaja.
      private Kupac  kup;           // Kupac koji kupuje.
  
      public Prodavac(Radnja r, double tMax)   // Inicijalizacija.
        { radnja = r; maxT = tMax; }

      public IPokretan Proteklo(double dt) {   // Sledei korak rada.
        if ((t -= dt) <= 0) {
          if (kup != null) {
            int i; Predmet p = null;
            for (i=0; i<radnja.polica.Kapacitet; i++)
              try {
                if ((p = radnja.polica[i]).Vr == kup.Vrs) break;
              } catch {}
            if (p != null) try { radnja.polica.Uzmi(i); } catch {}
            kup.Usluzen(p);
            kup = null;
          }
          try {
            kup = (Kupac)radnja.kupci.NaPrvi().DohvatiTek();
            radnja.kupci.Izbaci(kup);
            t = sluc.NextDouble() * maxT;
          } catch {}
        }
        return this;
      }
    } // class Prodavac
  }
} // class Radnja
