// Recnik2.cs - Klasa renika.

using System.IO; using System; using System.Text;

public class Recnik2 {
  
  private string imeDat;       // Ime datoteke za trajno uskladitavanje.
  private Rec prva;            // Prva re u reniku.
  private bool menjano;        // Da li je renik u memoriji menjan?
  
  private Recnik2(string ime) { imeDat = ime; }  // Nov prazan renik.
  
  public static Recnik2 Otvori(string imeDat) {  // Otvaranje renika.
    Recnik2 recnik = new Recnik2(imeDat);        // - prazan renik,
    StreamReader dat = null;
    try {
      dat = new StreamReader(imeDat);            // - popunjavanje
      while (true) {                             //   iz datoteke,
        string red = dat.ReadLine();
      if (red == null) break;
        string[] odr = red.Split('#');
        recnik.Dodaj(odr[0], odr[1]);
      }
    } catch (FileNotFoundException) {            // - ostaje prazan renik.
    } catch (IOException g) {
      Console.WriteLine(g); Environment.Exit(1); return null;
    } finally { if (dat != null) dat.Close(); }
    return recnik;
  }
 
  public Recnik2 Snimi() {                   // Snimanje renika u datoteku.
    StreamWriter dat = null;
    try {
      if (menjano) {
        dat = new StreamWriter(imeDat);
        for (Rec tekR=prva; tekR!=null; tekR=tekR.sled)
          for (Rec.Prevod tekP=tekR.prvi; tekP!=null; tekP=tekP.sled)
            dat.WriteLine(tekR.rec + "#" + tekP.prevod);
      }
      menjano = false;
    } catch (IOException g) {
      Console.WriteLine(g); Environment.Exit(1);
    } finally { if (dat != null) dat.Close(); }
    return this;
  }
  
  public Recnik2 Dodaj(string rec, string prevod) { // Dodavanje prevoda.
    Rec tek = prva, pret = null;
    while (tek!=null && rec.CompareTo(tek.rec)>0)
      { pret = tek; tek = tek.sled; }
    if (tek!=null && rec.Equals(tek.rec))
      tek.Dodaj(prevod, ref menjano);
    else {
      Rec nova = new Rec(rec, prevod, tek);
      if (pret == null) prva = nova; else pret.sled = nova;
      menjano = true;
    }
    return this;
  }
  
  public Recnik2 Brisi(string rec) {     // Brisanje rei sa svim prevodima.
    Rec tek = prva, pret = null;
    while (tek!=null && !rec.Equals(tek.rec))
      { pret = tek; tek = tek.sled; }
    if (tek != null) {
      if (pret == null) prva = tek.sled; else pret.sled = tek.sled;
      menjano = true;
    }
    return this;
  }
  
  public Recnik2 Brisi(string rec, string prevod) { // Brisanje jednog
    Rec tek = prva, pret = null;                    //   prevoda.
    while (tek!=null && !rec.Equals(tek.rec))
      { pret = tek; tek = tek.sled; }
    if (tek != null) {
      tek.Brisi(prevod, ref menjano);
      if (tek.prvi == null)
        { if (pret == null) prva = tek.sled; else pret.sled = tek.sled; }
    }
    return this;
  }
  
  public string[] Prevodi(string rec) {  // Dohvatanje svih prevoda rei.
    Rec tek = prva;
    while (tek!=null && !rec.Equals(tek.rec)) tek = tek.sled;
    if (tek != null) return tek.Prevodi(); else return null;
  }
  
 
  public override string ToString() {            // Tekstualni opis renika.
    StringBuilder sb = new StringBuilder();
    for (Rec tek=prva; tek!=null; tek=tek.sled)
      sb.Append(tek).Append('\n');
    return sb.ToString();
  }

  private class Rec {                            // KLASA ZA JEDNU RE.
    public string rec;                           // Tekst rei.
    public int brPrev = 1;                       // Broj prevoda.
    public Prevod prvi;                          // Prvi prevod.
    public Rec sled;                             // Sledea re u reniku.

    public Rec(string rec, string prevod, Rec sled) {// Stvaranje nove rei.
      this.rec = rec; this.sled = sled;
      prvi = new Prevod(prevod, null);
    }
    
    public Rec Dodaj(string prevod, ref bool menjano) {  // Dodavanje novog
      Prevod tek = prvi, pret = null;                    //   prevoda.
      while (tek!=null && prevod.CompareTo(tek.prevod)>0)
        { pret = tek; tek = tek.sled; }
      if (tek==null || !prevod.Equals(tek.prevod)) {
        Prevod novi = new Prevod(prevod, tek);
        if (pret == null) prvi = novi; else pret.sled = novi;
        brPrev++;
        menjano = true;
      }
      return this;
    }

    public Rec Brisi(string prevod, ref bool menjano) {  // Brisanje datog
      Prevod tek = prvi, pret = null;                    //   prevoda.
      while (tek!= null && prevod.CompareTo(tek.prevod)>0)
        { pret = tek; tek = tek.sled; }
      if (tek!=null && prevod.Equals(tek.prevod)) {
        if (pret == null) prvi = tek.sled; else pret.sled = tek.sled; 
        brPrev--;
        menjano = true;
      }
      return this;
    }
    
    public string[] Prevodi() {                  // Dohvatanje svih prevoda.
      string[] rez = new string [brPrev];
      int i = 0;
      for (Prevod tek=prvi; tek!=null; tek=tek.sled) rez[i++] = tek.prevod;
      return rez;
    }
    
    public override string ToString() { // Tekstualni opis rei s prevodima.
      StringBuilder sb = new StringBuilder();
      sb.Append(rec).Append(':');
      for (Prevod tek=prvi; tek!=null; tek=tek.sled) {
        sb.Append(' ').Append(tek.prevod)
          .Append(tek.sled!=null ? ',' : '.');
      }
      return sb.ToString();
    }
 
    public class Prevod {                        // KLASA ZA JEDAN PREVOD.
      public string prevod;                      // Tekst prevoda.
      public Prevod sled;                        // Sledei prevod.

      public Prevod(string prevod, Prevod sled) {// Stvaranje novog prevoda.   
        this.prevod = prevod; this.sled = sled;
      }
    } // class Prevod
  } // class Rec
} // class Recnik1
