// Skladiste2.cs - Klasa za uskladitavanje podataka.

namespace Skladista {
  using System.Threading;
  using System.Text;

  public class Skladiste {

    private static int posId = 0;  // Poslednji identifikator.
    private int id = ++posId;      // Identifikator skladita.
    private int[] niz;             // Niz za uskladitavanje podataka.
    private int ulaz, izlaz;       // Mesto stavljanja i uzimanja podatka.
    private int pun;               // Broj popunjenih mesta.

    public Skladiste(int kap) { niz = new int [kap]; }// Inicijalizacija.


    public void Stavi(int vredn) {                    // Stavljanje podatka.
      lock (this) {
        while (pun == niz.Length) Monitor.Wait(this);
        niz[ulaz++] = vredn; pun++;
        if (ulaz == niz.Length) ulaz = 0;
        Monitor.PulseAll(this);
      }
    }

    public int Uzmi() {                               // Uzimanje podatka.
      lock (this) {
        while (pun == 0) Monitor.Wait(this);
        int vredn = niz[izlaz++]; pun--;
        if (izlaz == niz.Length) izlaz = 0;
        Monitor.Pulse(this);
        return vredn;
      }
    }

    public override string ToString() {               // Tekstualni opis.
      StringBuilder s = new StringBuilder("Sklad " + id + ": ");
      lock (this) {
        for (int i=0; i<pun; i++)
          s.Append(niz[(izlaz+i) % niz.Length]).Append(' ');
      }
      return s.ToString();
    }
  }
}
