// Stablo.cs - Klasa ureenih binarnih stabala.

using System;

public class Stablo {
  public class Cvor {                           // vor stabla:
    public int broj;                            // - sadraj vora,
    public Cvor levo, desno;                    // - levo i desno podstablo,
    public Cvor(int b, Cvor l=null, Cvor d=null)// - konstruktor.
      { broj = b; levo = l; desno = d; }
  }

  private Cvor stb;                             // Koren stabla.
 
  public bool Prazno // Da li je stablo prazno?
    { get { return stb == null; } }
 
  public int Vel { get { return vel(stb); } }    // Broj vorova u stablu.

  public int Zbir { get { return zbir(stb); } }  // Zbir brojeva u stablu.
  
  public String NiskaKLD() { return niskaKLD(stb); } // Prefiksni opis.
  
  public String NiskaLKD() { return niskaLKD(stb); } // Infiksni opis.
  
  public String NiskaLDK() { return niskaLDK(stb); } // Postfiksni opis.
  
  public void Crtaj() { crtaj(stb, 0); }         // Grafiki prikaz stabla.

  public int Pojav(int b)                        // Broj pojavljivanja.
    { return pojav(stb, b); }

  public int Min { get {                         // Najmanji u stablu.
    if (stb == null) Environment.Exit(1);        //   (ne sme biti prazno)
    return min(stb);
  }}

  public int Max { get {                         // Najvei u stablu.
    if (stb == null) Environment.Exit(1);        //   (ne sme biti prazno)
    return max(stb);
  }}

  public Cvor Nadji(int b)                       // Traenje u stablu.
    { return nadji(stb, b); }

  public void Dodaj(int b) { dodaj(ref stb, b); }// Dodavanje u stablo.

  public void citaj(int n) {                     // itanje stabla.
    stb = null;
    for (int i=0; i<n; i++) Dodaj(Citaj.Int());
  }

  public void Prazni() { stb = null; }           // Pranjenje stabla.

  public void Izost(int b) { izost(ref stb, b); }// Izostavljanje iz stabla.

  public void Balans() { balans(stb); }          // Balansiranje stabla.

  // POMONE REKURZIVNE METODE:
  private static int vel(Cvor stb)               // Broj vorova u stablu.
    { return (stb != null) ? 1 + vel(stb.levo) + vel(stb.desno) : 0; }

  private static int zbir(Cvor stb)              // Zbir brojeva u stablu.
    { return (stb!=null) ? stb.broj+zbir(stb.levo)+zbir(stb.desno) : 0; }

  private static String niskaKLD(Cvor stb) {     // Prefiksni opis stabla.
    return (stb == null) ? "" :
      stb.broj + " " + niskaKLD(stb.levo) + niskaKLD(stb.desno);
  }

  private static String niskaLKD(Cvor stb) {     // Infiksni opis stabla.
    return (stb == null) ? "" :
      niskaLKD(stb.levo) + stb.broj + " " + niskaLKD(stb.desno);
  }

  private static String niskaLDK(Cvor stb) {     // Postfiksni opis stabla.
    return (stb == null) ? "" :
      niskaLDK(stb.levo) + niskaLDK(stb.desno) + stb.broj + " ";
  }
 
  private static void crtaj(Cvor stb, int nivo) {
    if (stb != null) {             // Grafiki prikaz stabla.
      crtaj(stb.desno, nivo+1);
      for (int i=0; i<nivo; i++) Console.Write("    ");
      Console.WriteLine(stb.broj);
      crtaj(stb.levo,  nivo+1);
    }
  }
                              // Broj pojavljivanja u stablu.
  private static int pojav(Cvor stb, int b) {
    return (stb != null) ? (stb.broj==b ? 1 : 0) +
        pojav(stb.levo,b) + pojav(stb.desno,b) : 0;
  }

  private static int min(Cvor stb)               // Najmanji u stablu.
    { return (stb.levo != null) ? min(stb.levo) : stb.broj; }

  private static int max(Cvor stb)               // Najvei u stablu.
    { return (stb.desno != null) ? max(stb.desno) : stb.broj; }

  private static Cvor nadji(Cvor stb, int b){    // Traenje u stablu.
    if (stb == null)   return null;
    if (stb.broj == b) return stb;
    if (stb.broj >  b) return nadji(stb.levo,  b);
                       return nadji(stb.desno, b);
  }

  private static Random sluc = new Random();

  private static void dodaj(ref Cvor stb, int b){// Dodavanje u stablo.
    if (stb == null)
      stb = new Cvor(b);
    else if (stb.broj > b)
       dodaj(ref stb.levo,  b);
    else if (stb.broj < b)
      dodaj(ref stb.desno, b);
    else if (sluc.NextDouble() < 0.5)
      dodaj(ref stb.levo,  b);
    else
      dodaj(ref stb.desno, b);
  }

  private static Cvor izost(ref Cvor stb, int b){// Izostavljanje iz stabla.
    if (stb != null) {
      if      (stb.broj > b)
        izost(ref stb.levo,  b);
      else if (stb.broj < b)
        izost(ref stb.desno, b);
      else if (stb.levo != null) {
        int m = max(stb.levo);
        stb.broj = m; izost(ref stb.levo,  m);
      } else if (stb.desno != null) {
        int m = min(stb.desno);
        stb.broj = m; izost(ref stb.desno, m);
      } else {
        stb = null;
      }
    }
    return stb;
  }
  private static void balans(Cvor stb) {         // Balansiranje stabla.
    if (stb != null) {
      int k = vel(stb.levo) - vel(stb.desno);
      for (; k>1; k-=2) {
        dodaj(ref stb.desno, stb.broj);
        stb.broj  = max(stb.levo );
        izost(ref stb.levo , stb.broj);
      }
      for (; k<-1; k+=2) {
        dodaj(ref stb.levo , stb.broj);
        stb.broj  = min(stb.desno);
        izost(ref stb.desno, stb.broj);
      }
      balans(stb.levo );
      balans(stb.desno);
    }
  }
}
