// Vektor2.cs - Klasa vektora s konkurentnom obradom.

using Usluge;
using System;
using System.Threading;

public class Vektor {

  private double[] niz;                         // Komponente vektora.

  public Vektor (int duz) { niz = new double [duz]; } // Konstruktor.
                                                
  public double this[int ind] {                 // Pristup komponenti:
    set {                                       // - postavljanje,
      if (ind < 0 || ind >= niz.Length)
        throw new Greska ("Nedozvoljen indeks!");
      niz[ind] = value;
    }
    get {                                       // - dohvatanje.
      if (ind < 0 || ind >= niz.Length)
        throw new Greska ("Nedozvoljen indeks!");
      return niz[ind];
    }
  }
 
  public override string ToString () {          // Tekstualni opis.
    string s = "(";
    for (int i=0; i<niz.Length; i++)
      { if (i > 0) s += ","; s += niz[i]; }
    return s + ")";
  }
  private class Sabirac {       // Pasivna klasa sabiraa (ne sadri nit):
    private double a, b, c;                     // - operandi i rezultat,
    public Sabirac (double p, double q)         // - konstruktor,
      { a = p; b = q; }
    public void Radi () { c = a + b; }          // - koristan rad,
    public double Rezultat { get { return c; }} // - dohvatanje rezultata.
  }
                                                // Zbir dva vektora:
  public static Vektor operator+ (Vektor v1, Vektor v2) {
    int n = v1.niz.Length, n2 = v2.niz.Length;
    if (n != n2) throw new Greska ("Neusaglasene duzine vektora!");
    Sabirac[] sab = new Sabirac [n];
    Thread[] nit = new Thread[n];
    for (int i=0; i<n; i++) {                   // - pokretanje niti,
      sab[i] = new Sabirac (v1.niz[i], v2.niz[i]);
      (nit[i] = new Thread (new ThreadStart (sab[i].Radi))).Start();
    }
    Vektor v = new Vektor (n);
    for (int i=0; i<n; i++) {                   // - sakupljanje rezultata.
        nit[i].Join(); v.niz[i] = sab[i].Rezultat; 
    }
    return v;
  }
  private class Mnozac {              // Aktivna klasa mnozaa (sadri nit):
    private double a, b, c;                     // - operandi i rezultat,
    private Thread nit;                         // - objekat niti,
    public Mnozac (double p, double q) {        // - konstruktor,
      a = p; b = q; 
      nit = new Thread (new ThreadStart (radi));
    }
    public void Pocni () { nit.Start (); }      // - pokretanje niti,
    private void radi () { c = a * b; }         // - sadraj niti,
    public void Sacekaj () { nit.Join (); }     // - ekanje zavretka niti,
    public double Rezultat { get { return c; }} // - dohvatanje rezultata.
  }
                                                // Skalarni proizvod:
  public static double SkalPro (Vektor v1, Vektor v2) {
    int n = v1.niz.Length, n2 = v2.niz.Length;
    if (n != n2) throw new Greska ("Neusaglasene duzine vektora!");
    Mnozac[] mno = new Mnozac [n];
    for (int i=0; i<n; i++)                     // - pokretanje niti,
      (mno[i] = new Mnozac (v1.niz[i], v2.niz[i])).Pocni ();
    double s = 0;
    for (int i=0; i<n; i++) {                   // - sakupljanje rezultata.
      mno[i].Sacekaj(); s += mno[i].Rezultat;
    }
    return s;
  }
 
  static void Main () {                         // Glavna metoda.
    while (true) {
      try {
        Console.Write ("n1? "); int n1 = Citaj.Int ();
    if (n1 <= 0) break;
        Console.Write ("v1? "); Vektor v1 = new Vektor (n1);
        for (int i=0; i<n1; v1[i++]=Citaj.Double ());
        Console.Write ("n2? "); int n2 = Citaj.Int ();
    if (n2 <= 0) break;
        Console.Write ("v2? "); Vektor v2 = new Vektor (n2);
        for (int i=0; i<n2; v2[i++]=Citaj.Double ());
        Console.WriteLine ("Zbir: " + (v1+v2));
        Console.WriteLine ("Skalpro: " + Vektor.SkalPro(v1,v2) + "\n");
      } catch (Greska g) {
        Console.WriteLine (g + "\n");
      }
    }
  }
}
